/***************************************************************************
                          cdl.cpp  -  description
                             -------------------
    begin                : Wed Oct 3 2001
    copyright            : (C) 2001 by Dannel Albert
    email                : dalbert@capitol-college.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cassert>
#include <iostream>
#include "cdl.h"
#include "unumstack.h"
#include "strstack.h"

using namespace std;

void SkipWhiteSpace(istream& ins);
bool GetAlphaNumeric(istream& ins, char& ch);

CanonicalDecimalList::CanonicalDecimalList()
{
  numInputs = 0;
  numCares = 0;
  numDontCares = 0;
  inputs = NULL;
  cares = NULL;
  dontCares = NULL;
}

CanonicalDecimalList::~CanonicalDecimalList()
{
  if (inputs)
  {
    delete[] inputs;
    inputs = NULL;
  }
  if (cares)
  {
    delete[] cares;
    cares = NULL;
  }
  if (dontCares)
  {
    delete[] dontCares;
    dontCares = NULL;
  }
}

ostream& operator<<(ostream& outs, const CanonicalDecimalList& cl)
{
  unsigned int i;
  outs << "f(";
  for(i=0; i<cl.numInputs; i++)
  {
    outs << cl.inputs[i];
    if (i != cl.numInputs-1)
      outs << ",";
  }
  outs << ") = m(";
  for(i=0; i<cl.numCares; i++)
  {
    outs << cl.cares[i];
    if (i != cl.numCares-1)
      outs << ",";
  }
  outs << ") + d(";
  for(i=0; i<cl.numDontCares; i++)
  {
    outs << cl.dontCares[i];
    if (i != cl.numDontCares-1)
      outs << ",";
  }
  outs << ")";

  return outs;
}

istream& operator>>(istream& ins, CanonicalDecimalList& cl)
{
#define BUFLENGTH 64
  UNumStack careStack, dontCareStack;
  StrStack inputStack;
  char ch;
  string str;
  char buf[BUFLENGTH];
  unsigned int index;
  bool done;

  SkipWhiteSpace(ins);
  ins.get(ch);
  if (ins.fail() || tolower(ch) != 'f')
    return ins;

  SkipWhiteSpace(ins);
  ins.get(ch);
  if (ins.fail() || ch != '(')
    return ins;

  SkipWhiteSpace(ins);
  index = 0;
  done = false;
  do
  {
    if (GetAlphaNumeric(ins, ch))
    {
        buf[index] = ch;
        index++;
        if (index >= BUFLENGTH-1) { return ins; }
    }
    else
    {
      switch(ch)
      {
      case ')':
        done = true;
      case ',':
        if (index != 0)
        {
          buf[index] = (char)NULL;
          str = buf;
          inputStack.Push(str);
          index = 0;
        }
        break;
      default:
        return ins;
      }
    }
    if (ins.fail())
      return ins;
  }
  while (!done);

  SkipWhiteSpace(ins);
  ins.get(ch);
  if (ins.fail() || ch != '=')
    return ins;

  SkipWhiteSpace(ins);
  ins.get(ch);
  if (ins.fail() || tolower(ch) != 'm')
    return ins;

  SkipWhiteSpace(ins);
  ins.get(ch);
  if (ins.fail() || ch != '(')
    return ins;

  SkipWhiteSpace(ins);
  index = 0;
  done = false;
  do
  {
    if (GetAlphaNumeric(ins, ch))
    {
        buf[index] = ch;
        index++;
        if (index >= BUFLENGTH-1) { return ins; }
    }
    else
    {
      switch(ch)
      {
      case ')':
        done = true;
      case ',':
        if (index != 0)
        {
          buf[index] = (char)NULL;
          careStack.Push(atoi(buf));
          index = 0;
        }
        break;
      default:
        return ins;
      }
    }
    if (ins.fail())
      return ins;
  }
  while (!done);

  SkipWhiteSpace(ins);
  ins.get(ch);
  if (ins.fail() || (ch != '+' && ch != '\n'))
    return ins;

  if (ch != '\n')
  {
    SkipWhiteSpace(ins);
    ins.get(ch);
    if (ins.fail() || tolower(ch) != 'd')
      return ins;

    SkipWhiteSpace(ins);
    ins.get(ch);
    if (ins.fail() || ch != '(')
      return ins;

    SkipWhiteSpace(ins);
    index = 0;
    done = false;
    do
    {
      if (GetAlphaNumeric(ins, ch))
      {
          buf[index] = ch;
          index++;
          if (index >= BUFLENGTH-1) { return ins; }
      }
      else
      {
        switch(ch)
        {
        case ')':
          done = true;
        case ',':
          if (index != 0)
          {
            buf[index] = (char)NULL;
            dontCareStack.Push(atoi(buf));
            index = 0;
          }
          break;
        default:
          return ins;
        }
      }
      if (ins.fail())
        return ins;
    }
    while (!done);
  }

  cl.numInputs = inputStack.Size();
  cl.numCares = careStack.Size();
  cl.numDontCares = dontCareStack.Size();

  if (cl.numInputs != 0)
  {
    cl.inputs = new string[cl.numInputs];
    assert(cl.inputs);
    for(int i=cl.numInputs-1; i>=0; i--)
    {
      cl.inputs[i] = inputStack.Pop();
    }
  }
  if (cl.numCares != 0)
  {
    cl.cares = new unsigned int[cl.numCares];
    assert(cl.cares);
    for(int i=cl.numCares-1; i>=0; i--)
    {
      cl.cares[i] = careStack.Pop();
    }
  }
  if (cl.numDontCares != 0)
  {
    cl.dontCares = new unsigned int[cl.numDontCares];
    assert(cl.dontCares);
    for(int i=cl.numDontCares-1; i>=0; i--)
    {
      cl.dontCares[i] = dontCareStack.Pop();
    }
  }

  return ins;
#undef BUFLENGTH
}

void SkipWhiteSpace(istream& ins)
{
  char ch;
  do
  { ins.get(ch); }
  while(ch == ' ' || ch == '\t');
  ins.unget();
}

bool GetAlphaNumeric(istream& ins, char& ch)
{
  ins.get(ch);
  if ((ch >= 'A' && ch <='Z') || (ch >= 'a' && ch <= 'z') ||
      (ch >='0' && ch <= '9'))
  {
    return true;
  }
  return false;
}