/***************************************************************************
                          quine.h  -  description
                             -------------------
    begin                : Thu Oct 4 2001
    copyright            : (C) 2001 by Dannel Albert
    email                : dalbert@capitol-college.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _QUINE_H_
#define _QUINE_H_

#include <iostream>
#include "globals.h"
#include "cdl.h"
#include "quinelist.h"
#include "quineterm.h"
#include "quinegroup.h"

class Quine
{
private:
  CanonicalDecimalList* cdl;
  QuineList* head;
  QuineList* tail;
  QuineGroup** primeImplicants;
  unsigned int numSteps;
  unsigned int numInputs;
  unsigned int numPI;
  bool solved;

public:
  // CONSTRUCTOR / DESTRUCTOR
  Quine();
  ~Quine();

  // ACCESSORS
  unsigned int GetNumInputs() const { return numInputs; }
  unsigned int GetNumSteps() const { return numSteps; }
  CanonicalDecimalList* GetCanonicalDecimalList() const { return cdl; }
  QuineGroup** GetPrimeImplicants(unsigned int& npi) const
  { npi = numPI; return primeImplicants; }
  bool Solve();

  friend std::ostream& operator<<(std::ostream& outs, const Quine& q);
  friend std::istream& operator>>(std::istream& ins, Quine& q);

private:
  bool Setup();
  void CreatePIList();
  void Insert(QuineList* ql);
};

std::ostream& operator<<(std::ostream& outs, const Quine& q);
std::istream& operator>>(std::istream& ins, Quine& q);

#endif