/***************************************************************************
                          quinegroup.cpp  -  description
                             -------------------
    begin                : Thu Oct 4 2001
    copyright            : (C) 2001 by Dannel Albert
    email                : dalbert@capitol-college.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cassert>
#include <cmath>
#include "quinegroup.h"

using namespace std;

QuineGroup::QuineGroup(unsigned int nb, QuineGroup* n, QuineGroup* p)
{
  head = tail = NULL;
  next = n;
  prev = p;
  diffBits = NULL;
  numBits = nb;
  numDiffBits = 0;
  numItems = 0;
  primeImplicant = 0;
  hasCombined = false;
  repeatCount = 0;
}

QuineGroup::~QuineGroup()
{
  QuineNode* nd = head;
  QuineNode* trash;
  while(nd != NULL)
  {
    trash = nd;
    nd = nd->GetNext();
    delete trash;
  }

  head = tail = NULL;
  next = prev = NULL;
  numItems = 0;
  primeImplicant = 0;
  hasCombined = false;
  repeatCount = 0;
}

unum* QuineGroup::GetDiffBits(unsigned int& numElements) const
{
  numElements = numDiffBits;
  return diffBits;
}

void QuineGroup::Insert(unum value)
{
  QuineNode* ndBefore = FindGreaterThan(value);
  QuineNode* nd;

  if (!numItems)  // empty list
  {
    nd = new QuineNode(value, numBits);
    head = tail = nd;
  }
  else if (ndBefore == head) // insert at beginning of list
  {
    nd = new QuineNode(value, numBits, head, NULL);
    head->SetPrev(nd);
    head = nd;
  }
  else if (ndBefore == NULL)  // insert at end of list
  {
    nd = new QuineNode(value, numBits, NULL, tail);
    tail->SetNext(nd);
    tail = nd;
  }
  else  // general case
  {
    nd = new QuineNode(value, numBits, ndBefore, ndBefore->GetPrev());
    (ndBefore->GetPrev())->SetNext(nd);
    ndBefore->SetPrev(nd);
  }
  numItems++;
}

bool QuineGroup::CalculateDiffBits()
{
  if (!numItems)
    return false;

  // numItems = 2^n
  // log(numItems) = n*log(2)
  // n = log(numItems) / log(2)
  // ^^^ doesn't work with integers well
  // factor out by 2 and count the # of 2's
  unsigned int i;
  for (i=1; i<numItems; i*=2)
  { numDiffBits++; }

  diffBits = new unsigned int[numDiffBits];
  assert(diffBits);

  unsigned int headData = head->GetData();
  for (i=0; i<numDiffBits; i++)
  {
    unsigned int numToSkip = (unsigned int)pow(2,i);
    diffBits[i] = ValueAt(numToSkip) - headData;
  }

  return true;
}

unum QuineGroup::ValueAt(unsigned int index) const
{
  if (index > numItems)
    return (unum)-1;

  QuineNode* nd = head;
  for (unsigned int i=0; i<index; i++)
    nd = nd->GetNext();

  return nd->GetData();
}

QuineNode* QuineGroup::FindGreaterThan(unum n)
{
  QuineNode* nd = head;

  if (!numItems)
    return NULL;

  while (nd != NULL && nd->GetData() < n)
  {
    nd = nd->GetNext();
  }

  return nd;
}

ostream& operator<<(ostream& outs, const QuineGroup& qg)
{
  if (!qg.numItems)
    return outs;

  QuineNode* nd = qg.head;
  for(unsigned int i=0; i<qg.numItems; i++)
  {
    outs << nd->GetData();
    if (i != qg.numItems-1)
      outs << ",";
    nd = nd->GetNext();
  }

  if (qg.diffBits)
  {
    outs << " (";
    for (unsigned int i=0; i<qg.numDiffBits; i++)
    {
      outs << qg.diffBits[i];
      if (i != qg.numDiffBits-1)
        outs << ",";
    }
    outs << ")";
  }

  if (qg.hasCombined)
  {
    outs << " *";
  }
  else if (qg.primeImplicant != 0)
  {
    outs << " PI#" << qg.primeImplicant;
  }

  return outs;
}