/***************************************************************************
                          quinegroup.h  -  description
                             -------------------
    begin                : Thu Oct 4 2001
    copyright            : (C) 2001 by Dannel Albert
    email                : dalbert@capitol-college.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _QUINEGROUP_H_
#define _QUINEGROUP_H_

#include <iostream>
#include "globals.h"
#include "quinenode.h"

class QuineGroup
{
private:
  QuineGroup* next;
  QuineGroup* prev;
  QuineNode* head;
  QuineNode* tail;
  unsigned int numBits;
  unsigned int numItems;
  unum* diffBits;
  unsigned int numDiffBits;
  unsigned int primeImplicant;
  unsigned int repeatCount;
  bool hasCombined;

public:
  // CONSTRUCTOR / DESTRUCTOR
  QuineGroup(unsigned int nb, QuineGroup* n=NULL, QuineGroup* p=NULL);
  ~QuineGroup();

  // ACCESSORS
  unsigned int Size() const { return numItems; }
  unum* GetDiffBits(unsigned int& numElements) const;
  QuineGroup* GetNext() const { return next; }
  QuineGroup* GetPrev() const { return prev; }
  unsigned int PrimeImplicant() const { return primeImplicant; }
  bool HasCombined() const { return hasCombined; }
  unsigned int GetRepeatCount() const { return repeatCount; }
  unsigned int GetNumBits() const { return numBits; }

  // MODIFIERS
  void SetNext(QuineGroup* n) { next = n; }
  void SetPrev(QuineGroup* p) { prev = p; }
  void SetPrimeImplicant(unsigned int pi) { primeImplicant = pi; }
  void SetHasCombined(bool hc) { hasCombined = hc; }
  void SetRepeatCount(unsigned int rc) { repeatCount = rc; }
  void SetNumBits(unsigned int nBits) { numBits = nBits; }

  void Insert(unum value);
  bool CalculateDiffBits();
  unum ValueAt(unsigned int index) const;

  friend std::ostream& operator<<(std::ostream& outs, const QuineGroup& qg);

private:
  QuineNode* FindGreaterThan(unum n);
};

std::ostream& operator<<(std::ostream& outs, const QuineGroup& qg);

#endif