/***************************************************************************
                          quinelist.h  -  description
                             -------------------
    begin                : Sat Oct 6 2001
    copyright            : (C) 2001 by Dannel Albert
    email                : dalbert@capitol-college.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _QUINELIST_H_
#define _QUINELIST_H_

#include <iostream>
#include "quineterm.h"

// *** NOTE ***
// Although QuineTerms are not created by the QuineList class,
// the QuineList class will delete all QuineTerms it contains
// when the QuineList destructor is called.
class QuineList
{
private:
  QuineTerm* head;
  QuineTerm* tail;
  QuineList* next;
  QuineList* prev;
  unsigned int numTerms;
  bool hasCombined;

public:
  // CONSTRUCTOR / DESTRUCTOR
  QuineList(QuineList* n=NULL, QuineList* p=NULL);
  ~QuineList();

  // ACCESSORS
  QuineList* GetNext() const { return next; }
  QuineList* GetPrev() const { return prev; }
  unsigned int Size() const { return numTerms; }
  bool TermsHaveCombined() const { return hasCombined; }

  // MODIFIERS
  void SetNext(QuineList* n) { next = n; }
  void SetPrev(QuineList* p) { prev = p; }
  void SetHasCombined(bool b) { hasCombined = b; }

  void Insert(QuineTerm* qt);
  QuineTerm* TermAt(unsigned int index) const;

  friend std::ostream& operator<<(std::ostream& outs, const QuineList& ql);
};

std::ostream& operator<<(std::ostream& outs, const QuineList& ql);

#endif