/***************************************************************************
                          quineterm.cpp  -  description
                             -------------------
    begin                : Sat Oct 6 2001
    copyright            : (C) 2001 by Dannel Albert
    email                : dalbert@capitol-college.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "quineterm.h"

using namespace std;

QuineTerm::QuineTerm(QuineTerm* n, QuineTerm* p)
{
  head = NULL;
  next = n;
  prev = p;
  numGroups = 0;
}

QuineTerm::~QuineTerm()
{
  QuineGroup* nd = head;
  QuineGroup* trash;
  while(nd != NULL)
  {
    trash = nd;
    nd = nd->GetNext();
    delete trash;
  }

  head = NULL;
  next = prev = NULL;
  numGroups = 0;
}

void QuineTerm::Insert(QuineGroup *qg)
{
  QuineGroup* ndBefore = FindGreaterThan(qg->ValueAt(0));

  if (!numGroups)  // empty list
  {
    head = tail = qg;
  }
  else if (ndBefore == head) // insert at beginning of list
  {
    qg->SetNext(head);
    head->SetPrev(qg);
    head = qg;
  }
  else if (ndBefore == NULL)  // insert at end of list
  {
    qg->SetPrev(tail);
    tail->SetNext(qg);
    tail = qg;
  }
  else  // general case
  {
    qg->SetNext(ndBefore);
    qg->SetPrev(ndBefore->GetPrev());
    (ndBefore->GetPrev())->SetNext(qg);
    ndBefore->SetPrev(qg);
  }
  numGroups++;
}

QuineGroup* QuineTerm::GroupAt(unsigned int index) const
{
  if (index > numGroups)
    return NULL;

  QuineGroup* qg = head;
  for(unsigned int i=0; i<index; i++)
    qg = qg->GetNext();

  return qg;
}

QuineGroup* QuineTerm::FindGreaterThan(unum n)
{
  QuineGroup* nd = head;

  if (!numGroups)
    return NULL;

  while (nd != NULL && nd->ValueAt(0) < n)
  {
    nd = nd->GetNext();
  }

  return nd;
}

ostream& operator<<(ostream& outs, const QuineTerm& qt)
{
  QuineGroup* qg = qt.head;
  for (unsigned int i=0; i<qt.numGroups; i++)
  {
    outs << *qg << endl;
    qg = qg->GetNext();
  }

  return outs;
}